/*
    SPDX-FileCopyrightText: 2012 Stefan Majewsky <majewsky@gmx.net>

    SPDX-License-Identifier: LGPL-2.0-only
*/

#ifndef KGTHEME_P_H
#define KGTHEME_P_H

#include "kgtheme.h"

#endif // KGTHEME_P_H
