/*
    SPDX-FileCopyrightText: 2010 Stefan Majewsky <majewsky@gmx.net>

    SPDX-License-Identifier: LGPL-2.0-only
*/

//This file is needed because the <KgSound> header needs to link to
//"../kgsound.h" instead of "../audio/kgsound.h".
#ifndef LIBKDEGAMES_KGSOUND_H
#define LIBKDEGAMES_KGSOUND_H

#include "audio/kgsound.h"

#endif
